/*
 * Copyright (c) 2023
 *
 * SPDX-License-Identifier: Apache-2.0
 */
#include <nrfx_clock.h>
#include <zephyr/drivers/spi.h>
#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>

#define BUFFER_SIZE 32768

// Enable logging
LOG_MODULE_REGISTER(main, CONFIG_LOG_DEFAULT_LEVEL);

// Get SPI configurations
static const struct device *spi_dev = DEVICE_DT_GET(DT_ALIAS(my_spi));
static const struct spi_cs_control cs = {
    .gpio = GPIO_DT_SPEC_GET_BY_IDX(DT_ALIAS(my_spi), cs_gpios, 0),
    .delay = 0,
};
static const struct spi_config spi_cfg = {
    .operation = SPI_WORD_SET(8) | SPI_TRANSFER_MSB | SPI_OP_MODE_MASTER,
    .frequency = 32000000,
    .slave = 0,
    .cs = &cs};

void anomaly_135(bool enable) {
  if (enable) {
    *(volatile uint32_t *)0x5000ac04 = 1;
  } else {
    *(volatile uint32_t *)0x5000ac04 = 0;
  }
}

int hfclk(const struct device *unused) {
  return nrfx_clock_divider_set(NRF_CLOCK_DOMAIN_HFCLK, NRF_CLOCK_HFCLK_DIV_1);
}
SYS_INIT(hfclk, EARLY, 0);

K_EVENT_DEFINE(spi_cb_event);

void spi_cb(const struct device *dev, int result, void *data) {
  anomaly_135(false);
  LOG_INF("  Callback: Got  result %d, sending event to main", result);
  k_event_set(&spi_cb_event, 0x0001);
}

// Function to read data from SPI
int read_with_cb(volatile uint8_t *rx_buf_array, size_t rx_buf_length) {
  // Configure TX buffer for transmission of initial value (0xF0 to start transaction)
  LOG_ERR("length: %d", rx_buf_length);
  uint8_t tx_buf_array[2] = {0xF0};
  const struct spi_buf tx_buf[1] = {{.buf = tx_buf_array, .len = 1}};
  const struct spi_buf_set tx = {.buffers = tx_buf, .count = 1};

  // Configure RX buffer for reception of buffer. Throwaway first byte
  uint8_t rx_buf_array_throwaway[1] = {0};
  const struct spi_buf rx_buf[2] = {
      {.buf = rx_buf_array_throwaway, .len = 1},
      {.buf = rx_buf_array, .len = rx_buf_length},
  };
  const struct spi_buf_set rx = {.buffers = rx_buf, .count = 2};

  // Perform transmission
  anomaly_135(true);
  return spi_transceive_cb(spi_dev, &spi_cfg, &tx, &rx, spi_cb, NULL);
}

void main(void) {
  int err = 0;
  LOG_INF("SPI mock firmware started");

  // Time for starting oscilloscope
  k_sleep(K_MSEC(4000));

  LOG_INF("Creating buffer!");
  uint8_t rx_buf[BUFFER_SIZE + 1] = {0};

  err = read_with_cb(rx_buf, BUFFER_SIZE); // Get data
  if (err < 0) {
    LOG_ERR("Got error %d from read_with_cb", err);
    return;
  }
  k_event_wait(&spi_cb_event, 0x0001, false, K_FOREVER);
  k_event_clear(&spi_cb_event, 0x0001);
  LOG_INF("Finished Processing test");
}
